==> koffice-trinity-3.5.13.2.tar.xz: koffice-trinity-3.5.13.2/kexi/plugins/forms/widgets/kexidbdatetimeedit.cpp <==
/* This file is part of the KDE project
   Copyright (C) 2005 Cedric Pasteur <cedric.pasteur@free.fr>
   Copyright (C) 2004-2005 Jaroslaw Staniek <js@iidea.pl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/

#include "kexidbdatetimeedit.h"

#include <tqtoolbutton.h>
#include <tqlayout.h>
#include <kpopupmenu.h>
#include <kdatepicker.h>
#include <kdatetbl.h>
#include <kexiutils/utils.h>

KexiDBDateTimeEdit::KexiDBDateTimeEdit(const TQDateTime &datetime, TQWidget *parent, const char *name)
 : TQWidget(parent, name), KexiFormDataItemInterface()
{
	m_invalidState = false;
	m_cleared = false;
	m_readOnly = false;

	m_dateEdit = new TQDateEdit(datetime.date(), this);
	m_dateEdit->setAutoAdvance(true);
	m_dateEdit->setSizePolicy(TQSizePolicy::Minimum, TQSizePolicy::MinimumExpanding);
//	m_dateEdit->setFixedWidth( TQFontMetrics(m_dateEdit->font()).width("8888-88-88___") );
	connect(m_dateEdit, TQT_SIGNAL(valueChanged(const TQDate&)), this, TQT_SLOT(slotValueChanged()));
	connect(m_dateEdit, TQT_SIGNAL(valueChanged(const TQDate&)), this, TQT_SIGNAL(dateTimeChanged()));

	TQToolButton* btn = new TQToolButton(this);
	btn->setText("...");
	btn->setFixedWidth( TQFontMetrics(btn->font()).width(" ... ") );
	btn->setPopupDelay(1); //1 ms

	m_timeEdit = new TQTimeEdit(datetime.time(), this);;
	m_timeEdit->setAutoAdvance(true);
	m_timeEdit->setSizePolicy(TQSizePolicy::Minimum, TQSizePolicy::MinimumExpanding);
	connect(m_timeEdit, TQT_SIGNAL(valueChanged(const TQTime&)), this, TQT_SLOT(slotValueChanged()));
	connect(m_timeEdit, TQT_SIGNAL(valueChanged(const TQTime&)), this, TQT_SIGNAL(dateTimeChanged()));

#ifdef TQDateTimeEditor_HACK
	m_dte_date = KexiUtils::findFirstChild<TQDateTimeEditor>(m_dateEdit, "TQDateTimeEditor");
	m_dte_time = KexiUtils::findFirstChild<TQDateTimeEditor>(m_timeEdit, "TQDateTimeEditor");
#else
	m_dte_date = 0;
#endif

	m_datePickerPopupMenu = new KPopupMenu(0, "date_popup");
	connect(m_datePickerPopupMenu, TQT_SIGNAL(aboutToShow()), this, TQT_SLOT(slotShowDatePicker()));
	m_datePicker = new KDatePicker(m_datePickerPopupMenu, TQDate::currentDate(), 0);

	KDateTable *dt = KexiUtils::findFirstChild<KDateTable>(m_datePicker, "KDateTable");
	if (dt)
		connect(dt, TQT_SIGNAL(tableClicked()), this, TQT_SLOT(acceptDate()));
	m_datePicker->setCloseButton(true);
	m_datePicker->installEventFilter(this);
	m_datePickerPopupMenu->insertItem(m_datePicker);
	btn->setPopup(m_datePickerPopupMenu);

	TQHBoxLayout* layout = new TQHBoxLayout(this);
	layout->addWidget(m_dateEdit, 0);
	layout->addWidget(btn, 0);
	layout->addWidget(m_timeEdit, 0);
	//layout->addStretch(1);

	setFocusProxy(m_dateEdit);
}

KexiDBDateTimeEdit::~KexiDBDateTimeEdit()
{
}

void KexiDBDateTimeEdit::setInvalidState(const TQString & /*! @todo paint this text: text*/)
{
	setEnabled(false);
	setReadOnly(true);
	m_invalidState = true;
//! @todo move this to KexiDataItemInterface::setInvalidStateInternal() ?
	if (focusPolicy() & TabFocus)
		setFocusPolicy(TQ_ClickFocus);
}

void
KexiDBDateTimeEdit::setEnabled(bool enabled)
{
	 // prevent the user from reenabling the widget when it is in invalid state
	if(enabled && m_invalidState)
		return;
	TQWidget::setEnabled(enabled);
}

void KexiDBDateTimeEdit::setValueInternal(const TQVariant &, bool )
{
	m_dateEdit->setDate(m_origValue.toDate());
	m_timeEdit->setTime(m_origValue.toTime());
}

TQVariant
KexiDBDateTimeEdit::value()
{
	return TQDateTime(m_dateEdit->date(), m_timeEdit->time());
}

bool KexiDBDateTimeEdit::valueIsNull()
{
	return !m_dateEdit->date().isValid() || m_dateEdit->date().isNull()
		|| !m_timeEdit->time().isValid() || m_timeEdit->time().isNull();
}

bool KexiDBDateTimeEdit::valueIsEmpty()
{
	return m_cleared;
}

bool KexiDBDateTimeEdit::isReadOnly() const
{
	//! @todo: data/time edit API has no readonly flag, 
	//!        so use event filter to avoid changes made by keyboard or mouse when m_readOnly==true
	return m_readOnly; //!isEnabled();
}

void KexiDBDateTimeEdit::setReadOnly(bool set)
{
	m_readOnly = set;
}

TQWidget*
KexiDBDateTimeEdit::widget()
{
	return m_dateEdit;
}

bool KexiDBDateTimeEdit::cursorAtStart()
{
#ifdef TQDateTimeEditor_HACK
	return m_dte_date && m_dateEdit->hasFocus() && m_dte_date->focusSection()==0;
#else
	return false;
#endif
}

bool KexiDBDateTimeEdit::cursorAtEnd()
{
#ifdef TQDateTimeEditor_HACK
	return m_dte_time && m_timeEdit->hasFocus()
		&& m_dte_time->focusSection()==int(m_dte_time->sectionCount()-1);
#else
	return false;
#endif
}

void KexiDBDateTimeEdit::clear()
{
	m_dateEdit->setDate(TQDate());
	m_timeEdit->setTime(TQTime());
	m_cleared = true;
}

void
KexiDBDateTimeEdit::slotValueChanged()
{
	m_cleared = false;
}

void
KexiDBDateTimeEdit::slotShowDatePicker()
{
	TQDate date = m_dateEdit->date();

	m_datePicker->setDate(date);
	m_datePicker->setFocus();
	m_datePicker->show();
	m_datePicker->setFocus();
}

void
KexiDBDateTimeEdit::acceptDate()
{
	m_dateEdit->setDate(m_datePicker->date());
	m_datePickerPopupMenu->hide();
}

bool
KexiDBDateTimeEdit::eventFilter(TQObject *o, TQEvent *e)
{
	if (o != m_datePicker)
		return false;

	switch (e->type()) {
		case TQEvent::Hide:
			m_datePickerPopupMenu->hide();
			break;
		case TQEvent::KeyPress:
		case TQEvent::KeyRelease: {
			TQKeyEvent *ke = (TQKeyEvent *)e;
			if (ke->key()==TQt::Key_Enter || ke->key()==TQt::Key_Return) {
				//accepting picker
				acceptDate();
				return true;
			}
			else if (ke->key()==TQt::Key_Escape) {
				//canceling picker
				m_datePickerPopupMenu->hide();
				return true;
			}
			else
				 m_datePickerPopupMenu->setFocus();
			break;
		}
		default:
			break;
	}
	return false;
}

TQDateTime
KexiDBDateTimeEdit::dateTime() const
{
	return TQDateTime(m_dateEdit->date(), m_timeEdit->time());
}

void
KexiDBDateTimeEdit::setDateTime(const TQDateTime &dt)
{
	m_dateEdit->setDate(dt.date());
	m_timeEdit->setTime(dt.time());
}

#include "kexidbdatetimeedit.moc"

==> koffice-trinity_14.0.11.orig.tar.xz: koffice-trinity-14.0.11/kexi/plugins/forms/widgets/kexidbdatetimeedit.cpp <==
/* This file is part of the KDE project
   Copyright (C) 2005 Cedric Pasteur <cedric.pasteur@free.fr>
   Copyright (C) 2004-2005 Jaroslaw Staniek <js@iidea.pl>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
*/

#include "kexidbdatetimeedit.h"

#include <tqtoolbutton.h>
#include <tqlayout.h>
#include <tdepopupmenu.h>
#include <kdatepicker.h>
#include <kdatetbl.h>
#include <kexiutils/utils.h>

KexiDBDateTimeEdit::KexiDBDateTimeEdit(const TQDateTime &datetime, TQWidget *parent, const char *name)
 : TQWidget(parent, name), KexiFormDataItemInterface()
{
	m_invalidState = false;
	m_cleared = false;
	m_readOnly = false;

	m_dateEdit = new TQDateEdit(datetime.date(), this);
	m_dateEdit->setAutoAdvance(true);
	m_dateEdit->setSizePolicy(TQSizePolicy::Minimum, TQSizePolicy::MinimumExpanding);
//	m_dateEdit->setFixedWidth( TQFontMetrics(m_dateEdit->font()).width("8888-88-88___") );
	connect(m_dateEdit, TQT_SIGNAL(valueChanged(const TQDate&)), this, TQT_SLOT(slotValueChanged()));
	connect(m_dateEdit, TQT_SIGNAL(valueChanged(const TQDate&)), this, TQT_SIGNAL(dateTimeChanged()));

	TQToolButton* btn = new TQToolButton(this);
	btn->setText("...");
	btn->setFixedWidth( TQFontMetrics(btn->font()).width(" ... ") );
	btn->setPopupDelay(1); //1 ms

	m_timeEdit = new TQTimeEdit(datetime.time(), this);;
	m_timeEdit->setAutoAdvance(true);
	m_timeEdit->setSizePolicy(TQSizePolicy::Minimum, TQSizePolicy::MinimumExpanding);
	connect(m_timeEdit, TQT_SIGNAL(valueChanged(const TQTime&)), this, TQT_SLOT(slotValueChanged()));
	connect(m_timeEdit, TQT_SIGNAL(valueChanged(const TQTime&)), this, TQT_SIGNAL(dateTimeChanged()));

#ifdef TQDateTimeEditor_HACK
	m_dte_date = KexiUtils::findFirstChild<TQDateTimeEditor>(m_dateEdit, "TQDateTimeEditor");
	m_dte_time = KexiUtils::findFirstChild<TQDateTimeEditor>(m_timeEdit, "TQDateTimeEditor");
#else
	m_dte_date = 0;
#endif

	m_datePickerPopupMenu = new TDEPopupMenu(0, "date_popup");
	connect(m_datePickerPopupMenu, TQT_SIGNAL(aboutToShow()), this, TQT_SLOT(slotShowDatePicker()));
	m_datePicker = new KDatePicker(m_datePickerPopupMenu, TQDate::currentDate(), 0);

	KDateTable *dt = KexiUtils::findFirstChild<KDateTable>(m_datePicker, "KDateTable");
	if (dt)
		connect(dt, TQT_SIGNAL(tableClicked()), this, TQT_SLOT(acceptDate()));
	m_datePicker->setCloseButton(true);
	m_datePicker->installEventFilter(this);
	m_datePickerPopupMenu->insertItem(m_datePicker);
	btn->setPopup(m_datePickerPopupMenu);

	TQHBoxLayout* layout = new TQHBoxLayout(this);
	layout->addWidget(m_dateEdit, 0);
	layout->addWidget(btn, 0);
	layout->addWidget(m_timeEdit, 0);
	//layout->addStretch(1);

	setFocusProxy(m_dateEdit);
}

KexiDBDateTimeEdit::~KexiDBDateTimeEdit()
{
}

void KexiDBDateTimeEdit::setInvalidState(const TQString & /*! @todo paint this text: text*/)
{
	setEnabled(false);
	setReadOnly(true);
	m_invalidState = true;
//! @todo move this to KexiDataItemInterface::setInvalidStateInternal() ?
	if (focusPolicy() & TabFocus)
		setFocusPolicy(TQ_ClickFocus);
}

void
KexiDBDateTimeEdit::setEnabled(bool enabled)
{
	 // prevent the user from reenabling the widget when it is in invalid state
	if(enabled && m_invalidState)
		return;
	TQWidget::setEnabled(enabled);
}

void KexiDBDateTimeEdit::setValueInternal(const TQVariant &, bool )
{
	m_dateEdit->setDate(m_origValue.toDate());
	m_timeEdit->setTime(m_origValue.toTime());
}

TQVariant
KexiDBDateTimeEdit::value()
{
	return TQDateTime(m_dateEdit->date(), m_timeEdit->time());
}

bool KexiDBDateTimeEdit::valueIsNull()
{
	return !m_dateEdit->date().isValid() || m_dateEdit->date().isNull()
		|| !m_timeEdit->time().isValid() || m_timeEdit->time().isNull();
}

bool KexiDBDateTimeEdit::valueIsEmpty()
{
	return m_cleared;
}

bool KexiDBDateTimeEdit::isReadOnly() const
{
	//! @todo: data/time edit API has no readonly flag, 
	//!        so use event filter to avoid changes made by keyboard or mouse when m_readOnly==true
	return m_readOnly; //!isEnabled();
}

void KexiDBDateTimeEdit::setReadOnly(bool set)
{
	m_readOnly = set;
}

TQWidget*
KexiDBDateTimeEdit::widget()
{
	return m_dateEdit;
}

bool KexiDBDateTimeEdit::cursorAtStart()
{
#ifdef TQDateTimeEditor_HACK
	return m_dte_date && m_dateEdit->hasFocus() && m_dte_date->focusSection()==0;
#else
	return false;
#endif
}

bool KexiDBDateTimeEdit::cursorAtEnd()
{
#ifdef TQDateTimeEditor_HACK
	return m_dte_time && m_timeEdit->hasFocus()
		&& m_dte_time->focusSection()==int(m_dte_time->sectionCount()-1);
#else
	return false;
#endif
}

void KexiDBDateTimeEdit::clear()
{
	m_dateEdit->setDate(TQDate());
	m_timeEdit->setTime(TQTime());
	m_cleared = true;
}

void
KexiDBDateTimeEdit::slotValueChanged()
{
	m_cleared = false;
}

void
KexiDBDateTimeEdit::slotShowDatePicker()
{
	TQDate date = m_dateEdit->date();

	m_datePicker->setDate(date);
	m_datePicker->setFocus();
	m_datePicker->show();
	m_datePicker->setFocus();
}

void
KexiDBDateTimeEdit::acceptDate()
{
	m_dateEdit->setDate(m_datePicker->date());
	m_datePickerPopupMenu->hide();
}

bool
KexiDBDateTimeEdit::eventFilter(TQObject *o, TQEvent *e)
{
	if (o != m_datePicker)
		return false;

	switch (e->type()) {
		case TQEvent::Hide:
			m_datePickerPopupMenu->hide();
			break;
		case TQEvent::KeyPress:
		case TQEvent::KeyRelease: {
			TQKeyEvent *ke = (TQKeyEvent *)e;
			if (ke->key()==TQt::Key_Enter || ke->key()==TQt::Key_Return) {
				//accepting picker
				acceptDate();
				return true;
			}
			else if (ke->key()==TQt::Key_Escape) {
				//canceling picker
				m_datePickerPopupMenu->hide();
				return true;
			}
			else
				 m_datePickerPopupMenu->setFocus();
			break;
		}
		default:
			break;
	}
	return false;
}

TQDateTime
KexiDBDateTimeEdit::dateTime() const
{
	return TQDateTime(m_dateEdit->date(), m_timeEdit->time());
}

void
KexiDBDateTimeEdit::setDateTime(const TQDateTime &dt)
{
	m_dateEdit->setDate(dt.date());
	m_timeEdit->setTime(dt.time());
}

#include "kexidbdatetimeedit.moc"
